@@ -2,7 +2,14 @@ import pytest
 from aiconfig.Config import AIConfigRuntime
 from aiconfig.util.config_utils import extract_override_settings
 
-from aiconfig.schema import AIConfig, ConfigMetadata, ExecuteResult, ModelMetadata, Prompt, PromptMetadata
+from aiconfig.schema import (
+    AIConfig,
+    ConfigMetadata,
+    ExecuteResult,
+    ModelMetadata,
+    Prompt,
+    PromptMetadata,
+)
 
 
 @pytest.fixture
@@ -61,7 +68,9 @@ def test_delete_nonexistent_model(ai_config: AIConfig):
     non_existent_model = "non_existent_model"
 
     # Ensure trying to delete a non-existent model raises an exception
-    with pytest.raises(Exception, match=f"Model '{non_existent_model}' does not exist."):
+    with pytest.raises(
+        Exception, match=f"Model '{non_existent_model}' does not exist."
+    ):
         ai_config.delete_model(non_existent_model)
 
 
@@ -156,7 +165,9 @@ def test_create_config_with_schema_version():
     """
     Test creating an AIConfig with a specific schema version.
     """
-    config_runtime = AIConfigRuntime.create("AIConfig with Schema", schema_version="v1")
+    config_runtime = AIConfigRuntime.create(
+        "AIConfig with Schema", schema_version="v1"
+    )
 
     config = config_runtime
 
@@ -183,7 +194,9 @@ def test_add_prompt_with_duplicate_name(ai_config_runtime: AIConfigRuntime):
     config.add_prompt("prompt1", prompt_data1)
 
     # Ensure adding a prompt with a duplicate name raises an exception
-    with pytest.raises(Exception, match=r"Prompt with name prompt1 already exists."):
+    with pytest.raises(
+        Exception, match=r"Prompt with name prompt1 already exists."
+    ):
         config.add_prompt("prompt1", prompt_data2)
 
 
@@ -201,7 +214,10 @@ def test_update_nonexistent_prompt(ai_config_runtime: AIConfigRuntime):
     )
 
     # Ensure updating a nonexistent prompt raises an exception
-    with pytest.raises(IndexError, match=f"Prompt '{nonexistent_prompt_name}' not found in config"):
+    with pytest.raises(
+        IndexError,
+        match=f"Prompt '{nonexistent_prompt_name}' not found in config",
+    ):
         config.update_prompt(nonexistent_prompt_name, prompt_data)
 
 
@@ -213,11 +229,15 @@ def test_delete_nonexistent_prompt(ai_config_runtime: AIConfigRuntime):
     prompt_name = "nonexistent_prompt"
 
     # Ensure deleting a nonexistent prompt raises an exception
-    with pytest.raises(IndexError, match=f"Prompt '{prompt_name}' not found in config"):
+    with pytest.raises(
+        IndexError, match=f"Prompt '{prompt_name}' not found in config"
+    ):
         config.delete_prompt(prompt_name)
 
 
-def test_get_metadata_with_nonexistent_prompt(ai_config_runtime: AIConfigRuntime):
+def test_get_metadata_with_nonexistent_prompt(
+    ai_config_runtime: AIConfigRuntime,
+):
     """
     Test the retrieval of metadata from a non-existent prompt.
     """
@@ -225,7 +245,9 @@ def test_get_metadata_with_nonexistent_prompt(ai_config_runtime: AIConfigRuntime
     prompt_name = "nonexistent_prompt"
 
     # Ensure that attempting to retrieve metadata for a non-existent prompt raises an exception
-    with pytest.raises(IndexError, match=f"Prompt '{prompt_name}' not found in config"):
+    with pytest.raises(
+        IndexError, match=f"Prompt '{prompt_name}' not found in config"
+    ):
         config.get_metadata(prompt_name)
 
 
@@ -247,7 +269,9 @@ def test_load_saved_config(tmp_path):
     config_runtime = AIConfigRuntime.create("My AIConfig")
 
     # Create a configuration-level parameter
-    config_runtime.set_parameter("config_param", "config_value", prompt_name=None)
+    config_runtime.set_parameter(
+        "config_param", "config_value", prompt_name=None
+    )
 
     # Create a sample prompt for testing
     prompt_data = Prompt(
@@ -258,7 +282,9 @@ def test_load_saved_config(tmp_path):
     config_runtime.add_prompt("prompt1", prompt_data)
 
     # Set a prompt-level parameter
-    config_runtime.set_parameter("prompt_param", "prompt_value", prompt_name="prompt1")
+    config_runtime.set_parameter(
+        "prompt_param", "prompt_value", prompt_name="prompt1"
+    )
 
     json_config_filepath = tmp_path / "my_aiconfig.json"
     config_runtime.save(json_config_filepath)
@@ -267,10 +293,14 @@ def test_load_saved_config(tmp_path):
 
     # Ensure the loaded AIConfig contains the expected data
     assert loaded_config.name == "My AIConfig"
-    assert loaded_config.metadata.parameters == {"config_param": "config_value"}
+    assert loaded_config.metadata.parameters == {
+        "config_param": "config_value"
+    }
     assert "prompt1" in loaded_config.prompt_index
     assert loaded_config.prompt_index["prompt1"].metadata is not None
-    assert loaded_config.prompt_index["prompt1"].metadata.parameters == {"prompt_param": "prompt_value"}
+    assert loaded_config.prompt_index["prompt1"].metadata.parameters == {
+        "prompt_param": "prompt_value"
+    }
 
 
 def test_set_config_name(ai_config_runtime: AIConfigRuntime):
@@ -295,7 +325,9 @@ def test_get_prompt_existing(ai_config_runtime: AIConfigRuntime):
     assert retrieved_prompt == prompt
 
 
-def test_get_prompt_after_deleting_previous(ai_config_runtime: AIConfigRuntime):
+def test_get_prompt_after_deleting_previous(
+    ai_config_runtime: AIConfigRuntime,
+):
     prompt1 = Prompt(
         name="GreetingPrompt",
         input="Hello, how are you?",
@@ -314,7 +346,9 @@ def test_get_prompt_after_deleting_previous(ai_config_runtime: AIConfigRuntime):
 
 
 def test_get_prompt_nonexistent(ai_config_runtime: AIConfigRuntime):
-    with pytest.raises(IndexError, match=r"Prompt 'GreetingPrompt' not found in config"):
+    with pytest.raises(
+        IndexError, match=r"Prompt 'GreetingPrompt' not found in config"
+    ):
         ai_config_runtime.get_prompt("GreetingPrompt")
 
 
@@ -324,27 +358,35 @@ def test_update_model_for_ai_config(ai_config_runtime: AIConfigRuntime):
     model_name = "testmodel"
     settings = {"topP": 0.9}
     ai_config_runtime.update_model(model_name, settings)
-    pytest.warns(match=f"No prompt name was given to update the model name to '{model_name}'.")
+    pytest.warns(
+        match=f"No prompt name was given to update the model name to '{model_name}'."
+    )
     assert ai_config_runtime.metadata.models is not None
     assert ai_config_runtime.metadata.models[model_name] == settings
 
     # Existing model name, no settings --> no-op
     ai_config_runtime.update_model(model_name)
-    pytest.warns(match=f"No prompt name was given to update the model name to '{model_name}'.")
+    pytest.warns(
+        match=f"No prompt name was given to update the model name to '{model_name}'."
+    )
     assert ai_config_runtime.metadata.models is not None
     assert ai_config_runtime.metadata.models[model_name] == settings
 
     # Existing model name, new settings --> update
     new_settings = {"topP": 0.75}
     ai_config_runtime.update_model(model_name, new_settings)
-    pytest.warns(match=f"No prompt name was given to update the model name to '{model_name}'.")
+    pytest.warns(
+        match=f"No prompt name was given to update the model name to '{model_name}'."
+    )
     assert ai_config_runtime.metadata.models is not None
     assert ai_config_runtime.metadata.models[model_name] == new_settings
 
     # New model name, no settings --> update
     new_model_name = "testmodel_without_settings"
     ai_config_runtime.update_model(new_model_name)
-    pytest.warns(match=f"No prompt name was given to update the model name to '{new_model_name}'.")
+    pytest.warns(
+        match=f"No prompt name was given to update the model name to '{new_model_name}'."
+    )
     assert ai_config_runtime.metadata.models is not None
     assert ai_config_runtime.metadata.models[new_model_name] == {}
 
@@ -359,21 +401,27 @@ def test_update_model_for_prompt(ai_config_runtime: AIConfigRuntime):
     ai_config_runtime.update_model(model_name, settings, prompt1.name)
     prompt = ai_config_runtime.get_prompt(prompt1.name)
     assert prompt.metadata is not None
-    assert prompt.metadata.model == ModelMetadata(name=model_name, settings=settings)
+    assert prompt.metadata.model == ModelMetadata(
+        name=model_name, settings=settings
+    )
 
     # New model name, no settings --> update name only
     new_model_name = "testmodel_new_name"
     ai_config_runtime.update_model(new_model_name, None, prompt1.name)
     prompt = ai_config_runtime.get_prompt(prompt1.name)
     assert prompt.metadata is not None
-    assert prompt.metadata.model == ModelMetadata(name=new_model_name, settings=settings)
+    assert prompt.metadata.model == ModelMetadata(
+        name=new_model_name, settings=settings
+    )
 
     # Same model name, new settings --> update settings only
     settings = {"topP": 0.9}
     ai_config_runtime.update_model(new_model_name, settings, prompt1.name)
     prompt = ai_config_runtime.get_prompt(prompt1.name)
     assert prompt.metadata is not None
-    assert prompt.metadata.model == ModelMetadata(name=new_model_name, settings=settings)
+    assert prompt.metadata.model == ModelMetadata(
+        name=new_model_name, settings=settings
+    )
 
     # New name, no settings, prompt with model name as string --> update
     prompt2 = Prompt(
@@ -386,7 +434,9 @@ def test_update_model_for_prompt(ai_config_runtime: AIConfigRuntime):
     ai_config_runtime.update_model(new_name_again, None, prompt2.name)
     prompt = ai_config_runtime.get_prompt(prompt2.name)
     assert prompt.metadata is not None
-    assert prompt.metadata.model == ModelMetadata(name=new_name_again, settings={})
+    assert prompt.metadata.model == ModelMetadata(
+        name=new_name_again, settings={}
+    )
 
     # New name, no settings, prompt with metadata but no model --> update
     tags = ["my_fancy_tags"]
@@ -404,7 +454,9 @@ def test_update_model_for_prompt(ai_config_runtime: AIConfigRuntime):
     assert prompt.metadata.tags == tags
 
 
-def test_update_model_with_invalid_arguments(ai_config_runtime: AIConfigRuntime):
+def test_update_model_with_invalid_arguments(
+    ai_config_runtime: AIConfigRuntime,
+):
     """Test trying to update model with invalid arguments."""
     with pytest.raises(
         ValueError,
@@ -416,7 +468,9 @@ def test_update_model_with_invalid_arguments(ai_config_runtime: AIConfigRuntime)
         ValueError,
         match=r"Cannot update model. There are two things you are trying:",
     ):
-        ai_config_runtime.update_model(model_name=None, settings={"top": 0.9}, prompt_name=None)
+        ai_config_runtime.update_model(
+            model_name=None, settings={"top": 0.9}, prompt_name=None
+        )
 
 
 def test_set_and_delete_metadata_ai_config(ai_config_runtime: AIConfigRuntime):
@@ -430,7 +484,9 @@ def test_set_and_delete_metadata_ai_config(ai_config_runtime: AIConfigRuntime):
     assert hasattr(ai_config_runtime.get_metadata(), "testkey") is False
 
 
-def test_set_and_delete_metadata_ai_config_prompt(ai_config_runtime: AIConfigRuntime):
+def test_set_and_delete_metadata_ai_config_prompt(
+    ai_config_runtime: AIConfigRuntime,
+):
     """Test deleting a non-existent metadata key at the AIConfig level."""
     prompt = Prompt(
         name="GreetingPrompt",
@@ -440,14 +496,24 @@ def test_set_and_delete_metadata_ai_config_prompt(ai_config_runtime: AIConfigRun
     ai_config_runtime.add_prompt(prompt.name, prompt)
     ai_config_runtime.set_metadata("testkey", "testvalue", "GreetingPrompt")
 
-    assert ai_config_runtime.get_prompt("GreetingPrompt").metadata.testkey == "testvalue"
+    assert (
+        ai_config_runtime.get_prompt("GreetingPrompt").metadata.testkey
+        == "testvalue"
+    )
 
     ai_config_runtime.delete_metadata("testkey", "GreetingPrompt")
 
-    assert hasattr(ai_config_runtime.get_prompt("GreetingPrompt").metadata, "testkey") is False
+    assert (
+        hasattr(
+            ai_config_runtime.get_prompt("GreetingPrompt").metadata, "testkey"
+        )
+        is False
+    )
 
 
-def test_add_output_existing_prompt_no_overwrite(ai_config_runtime: AIConfigRuntime):
+def test_add_output_existing_prompt_no_overwrite(
+    ai_config_runtime: AIConfigRuntime,
+):
     """Test adding an output to an existing prompt without overwriting."""
     prompt = Prompt(
         name="GreetingPrompt",
@@ -459,7 +525,9 @@ def test_add_output_existing_prompt_no_overwrite(ai_config_runtime: AIConfigRunt
         output_type="execute_result",
         execution_count=0,
         data="test output",
-        metadata={"raw_response": {"role": "assistant", "content": "test output"}},
+        metadata={
+            "raw_response": {"role": "assistant", "content": "test output"}
+        },
     )
     ai_config_runtime.add_output("GreetingPrompt", test_result)
 
@@ -469,23 +537,34 @@ def test_add_output_existing_prompt_no_overwrite(ai_config_runtime: AIConfigRunt
         output_type="execute_result",
         execution_count=0,
         data="test output",
-        metadata={"raw_response": {"role": "assistant", "content": "test output for second time"}},
+        metadata={
+            "raw_response": {
+                "role": "assistant",
+                "content": "test output for second time",
+            }
+        },
     )
 
     ai_config_runtime.add_output("GreetingPrompt", test_result2)
-    assert ai_config_runtime.get_latest_output("GreetingPrompt") == test_result2
+    assert (
+        ai_config_runtime.get_latest_output("GreetingPrompt") == test_result2
+    )
 
     ai_config_runtime.delete_output("GreetingPrompt")
     assert ai_config_runtime.get_latest_output("GreetingPrompt") == None
 
 
-def test_add_outputs_existing_prompt_no_overwrite(ai_config_runtime: AIConfigRuntime):
+def test_add_outputs_existing_prompt_no_overwrite(
+    ai_config_runtime: AIConfigRuntime,
+):
     """Test adding outputs to an existing prompt without overwriting."""
     original_result = ExecuteResult(
         output_type="execute_result",
         execution_count=0,
         data="original result",
-        metadata={"raw_response": {"role": "assistant", "content": "original result"}},
+        metadata={
+            "raw_response": {"role": "assistant", "content": "original result"}
+        },
     )
     prompt = Prompt(
         name="GreetingPrompt",
@@ -495,33 +574,48 @@ def test_add_outputs_existing_prompt_no_overwrite(ai_config_runtime: AIConfigRun
     )
     ai_config_runtime.add_prompt(prompt.name, prompt)
 
-    assert ai_config_runtime.get_latest_output("GreetingPrompt") == original_result
+    assert (
+        ai_config_runtime.get_latest_output("GreetingPrompt")
+        == original_result
+    )
 
     test_result1 = ExecuteResult(
         output_type="execute_result",
         execution_count=0,
         data="test output 1",
-        metadata={"raw_response": {"role": "assistant", "content": "test output 1"}},
+        metadata={
+            "raw_response": {"role": "assistant", "content": "test output 1"}
+        },
     )
     test_result2 = ExecuteResult(
         output_type="execute_result",
         execution_count=0,
         data="test output 2",
-        metadata={"raw_response": {"role": "assistant", "content": "test output 2"}},
+        metadata={
+            "raw_response": {"role": "assistant", "content": "test output 2"}
+        },
+    )
+    ai_config_runtime.add_outputs(
+        "GreetingPrompt", [test_result1, test_result2]
     )
-    ai_config_runtime.add_outputs("GreetingPrompt", [test_result1, test_result2])
 
-    assert ai_config_runtime.get_latest_output("GreetingPrompt") == test_result2
+    assert (
+        ai_config_runtime.get_latest_output("GreetingPrompt") == test_result2
+    )
     assert prompt.outputs == [original_result, test_result1, test_result2]
 
 
-def test_add_outputs_existing_prompt_with_overwrite(ai_config_runtime: AIConfigRuntime):
+def test_add_outputs_existing_prompt_with_overwrite(
+    ai_config_runtime: AIConfigRuntime,
+):
     """Test adding outputs to an existing prompt with overwriting."""
     original_result = ExecuteResult(
         output_type="execute_result",
         execution_count=0,
         data="original result",
-        metadata={"raw_response": {"role": "assistant", "content": "original result"}},
+        metadata={
+            "raw_response": {"role": "assistant", "content": "original result"}
+        },
     )
     prompt = Prompt(
         name="GreetingPrompt",
@@ -531,23 +625,34 @@ def test_add_outputs_existing_prompt_with_overwrite(ai_config_runtime: AIConfigR
     )
     ai_config_runtime.add_prompt(prompt.name, prompt)
 
-    assert ai_config_runtime.get_latest_output("GreetingPrompt") == original_result
+    assert (
+        ai_config_runtime.get_latest_output("GreetingPrompt")
+        == original_result
+    )
 
     test_result1 = ExecuteResult(
         output_type="execute_result",
         execution_count=0,
         data="test output 1",
-        metadata={"raw_response": {"role": "assistant", "content": "test output 1"}},
+        metadata={
+            "raw_response": {"role": "assistant", "content": "test output 1"}
+        },
     )
     test_result2 = ExecuteResult(
         output_type="execute_result",
         execution_count=0,
         data="test output 2",
-        metadata={"raw_response": {"role": "assistant", "content": "test output 2"}},
+        metadata={
+            "raw_response": {"role": "assistant", "content": "test output 2"}
+        },
+    )
+    ai_config_runtime.add_outputs(
+        "GreetingPrompt", [test_result1, test_result2], True
     )
-    ai_config_runtime.add_outputs("GreetingPrompt", [test_result1, test_result2], True)
 
-    assert ai_config_runtime.get_latest_output("GreetingPrompt") == test_result2
+    assert (
+        ai_config_runtime.get_latest_output("GreetingPrompt") == test_result2
+    )
     assert prompt.outputs == [test_result1, test_result2]
 
 
@@ -574,13 +679,17 @@ def test_add_undefined_outputs_to_prompt(ai_config_runtime: AIConfigRuntime):
         ai_config_runtime.add_outputs("GreetingPrompt", [], True)
 
 
-def test_add_output_existing_prompt_overwrite(ai_config_runtime: AIConfigRuntime):
+def test_add_output_existing_prompt_overwrite(
+    ai_config_runtime: AIConfigRuntime,
+):
     """Test adding an output to an existing prompt with overwriting."""
     original_output = ExecuteResult(
         output_type="execute_result",
         execution_count=0,
         data="original output",
-        metadata={"raw_response": {"role": "assistant", "content": "original output"}},
+        metadata={
+            "raw_response": {"role": "assistant", "content": "original output"}
+        },
     )
     prompt = Prompt(
         name="GreetingPrompt",
@@ -590,16 +699,24 @@ def test_add_output_existing_prompt_overwrite(ai_config_runtime: AIConfigRuntime
     )
     ai_config_runtime.add_prompt(prompt.name, prompt)
     # check that the original_output is there
-    assert ai_config_runtime.get_latest_output("GreetingPrompt") == original_output
+    assert (
+        ai_config_runtime.get_latest_output("GreetingPrompt")
+        == original_output
+    )
     expected_output = ExecuteResult(
         output_type="execute_result",
         execution_count=0,
         data="original output",
-        metadata={"raw_response": {"role": "assistant", "content": "original output"}},
+        metadata={
+            "raw_response": {"role": "assistant", "content": "original output"}
+        },
     )
     # overwrite the original_output
     ai_config_runtime.add_output("GreetingPrompt", expected_output, True)
-    assert ai_config_runtime.get_latest_output("GreetingPrompt") == expected_output
+    assert (
+        ai_config_runtime.get_latest_output("GreetingPrompt")
+        == expected_output
+    )
 
 
 def test_add_undefined_output_to_prompt(ai_config_runtime: AIConfigRuntime):
@@ -628,17 +745,25 @@ def test_extract_override_settings(ai_config_runtime: AIConfigRuntime):
     initial_settings = {"topP": 0.9}
 
     # Test Case 1: No global setting, Expect an override
-    override = extract_override_settings(ai_config_runtime, initial_settings, "testmodel")
+    override = extract_override_settings(
+        ai_config_runtime, initial_settings, "testmodel"
+    )
     assert override == {"topP": 0.9}
 
     # Test Case 2: Global Settings differ, expect override
     ai_config_runtime.add_model("testmodel", {"topP": 0.8})
-    override = extract_override_settings(ai_config_runtime, initial_settings, "testmodel")
+    override = extract_override_settings(
+        ai_config_runtime, initial_settings, "testmodel"
+    )
     assert override == {"topP": 0.9}
 
     # Test Case 3: Global Settings match settings, expect no override
-    ai_config_runtime.update_model(model_name="testmodel", settings={"topP": 0.9})
-    override = extract_override_settings(ai_config_runtime, initial_settings, "testmodel")
+    ai_config_runtime.update_model(
+        model_name="testmodel", settings={"topP": 0.9}
+    )
+    override = extract_override_settings(
+        ai_config_runtime, initial_settings, "testmodel"
+    )
     assert override == {}
 
     # Test Case 4: Global settings defined and empty settings defined. Expect no override
